/*
**  Copyright (C) 1993-1996, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/

#include "osconfig.h"

#include <stdio.h>
#include <string.h>

#ifdef HAVE_STDLIB_H
#ifndef  _BCB4
/* workaround for bug in Borland C++ Builder 4 */
BEGIN_EXTERN_C
#endif
#include <stdlib.h>
#ifndef  _BCB4
END_EXTERN_C
#endif
#endif

#ifdef HAVE_LIBC_H
#include <libc.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#include "dispuser.h"
#include "dcuid.h"    /* for dcmtk version name */

static char rcsid[] = "$dcmtk: sendimg v"
  OFFIS_DCMTK_VERSION " " OFFIS_DCMTK_RELEASEDATE " $";

#ifndef MAXPATHLEN
#define MAXPATHLEN 2048 /* should be sufficient for most cases */
#endif

#ifdef HAVE_GETCWD
#ifndef HAVE_PROTOTYPE_GETCWD
BEGIN_EXTERN_C
char *getcwd(char *buf, size_t size);
END_EXTERN_C
#endif
#endif

#ifdef HAVE_GETWD
#ifndef HAVE_PROTOTYPE_GETWD
BEGIN_EXTERN_C
char *getwd(char *buf);
END_EXTERN_C
#endif
#endif

int main(int argc, char **argv)
{
  int i;
  char cwd[MAXPATHLEN];
  char fname[MAXPATHLEN];
  FILE *file;

  if (argc < 2)
  {
    fprintf(stderr, "%s\n\nusage: %s imagefile ...\n"
      "  requests CTNdisp to display image(s)\n", 
      rcsid, argv[0]);
    exit(1);
  }

  DSU_setDisplayMode(1);
  if (!DSU_initDisplay(DSU_CTNUser))
  {
    fprintf(stderr, "Sorry, CTNdisp not found.\n");
    exit(0);
  }

#if defined(HAVE_GETCWD)
  getcwd(cwd, MAXPATHLEN);
#elif defined(HAVE_GETWD)
  getwd(cwd);
#else
  strcpy(cwd,""); /* we cannot determine our working directory */
#endif

  for (i=1; i<argc; i++)
  {
    if (NULL !=(file=fopen(argv[i],"rb")))
    {
      fclose(file);
    } else {
      fprintf(stderr,"cannot read file %s\n",argv[i]);
      exit(1);
    }

   /*
    * The display program could be running from
    * another directory thus relative path names
    * will often cause problems.  Convert relative
    * pathnames into absolute.
    */

    if ((argv[i][0] != '/')&&(strlen(cwd)>0)) 
    {
      sprintf(fname, "%s/%s", cwd, argv[i]);
    } else {
      strcpy(fname, argv[i]);
    }

    /* ask CTNdisplay to display the image */
    printf("request display: %s\n", fname);
    DSU_displayImage(DSU_NoAssociation, fname);
  }
  return 0;
}

