/*
 *
 *  Copyright (C) 1996-2000, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Modality Worklist Server Central Test Node (ctn) Main Program
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 2000/03/08 16:37:45 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/apps/wlistctn.cc,v $
 *  CVS/RCS Revision: $Revision: 1.23 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#ifdef HAVE_STDLIB_H
#ifndef  _BCB4
/* workaround for bug in Borland C++ Builder 4 */
BEGIN_EXTERN_C
#endif
#include <stdlib.h>
#ifndef  _BCB4
END_EXTERN_C
#endif
#endif

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#include <time.h>
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#include <errno.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#ifdef HAVE_SYS_ERRNO_H
#include <sys/errno.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "wlistctn.h"
#include "dimse.h"
#include "sce.h"
#ifndef NODISPLAY
#include "dispuser.h"
#endif
#include "wrklstdb.h"
#include "dcdict.h"
#include "dcdebug.h"
#include "cmdlnarg.h"
#include "dcuid.h"    /* for dcmtk version name */

static char rcsid[] = "$dcmtk: wlistctn v"
  OFFIS_DCMTK_VERSION " " OFFIS_DCMTK_RELEASEDATE " $";

#define APPLICATIONTITLE	"WLISTCTN"

typedef struct _ctn_processslot CTN_ProcessSlot;

struct _ctn_processslot {
	DIC_NODENAME		peerName;
	DIC_AE			callingAETitle;
	DIC_AE			calledAETitle;
	int			processId;
	time_t			startTime;
	OFBool			hasStorageAbility;
	CTN_ProcessSlot		*next;
};

typedef struct {
	int			pcnt;	
	CTN_ProcessSlot		*plist;
} CTN_ProcessTable;

static CTN_ProcessTable	processTable;

const char *supportedAbstractSyntaxes[] = {
    UID_VerificationSOPClass, 
    UID_FINDModalityWorklistInformationModel
};

int numberOfSupportedAbstractSyntaxes = DIM_OF(supportedAbstractSyntaxes);

/* Reasons for refusing association */
typedef enum {
	CTN_TooManyAssociations,
	CTN_CannotFork,
	CTN_BadAppContext,
	CTN_BadAEPeer,
	CTN_BadAEService,
	CTN_NoReason
} CTN_RefuseReason;



const char* progname = NULL;
const char* baseWorklistDBPath = "/home/www/wlist";

unsigned long     opt_port = 0;
int               opt_refuseAssociation = 0;
int               opt_rejectWithoutImplementationUID = 0;
unsigned long     opt_sleepAfterFind = 0;
unsigned long     opt_sleepDuringFind = 0;
unsigned long     opt_maxPDU = ASC_DEFAULTMAXPDU;
E_TransferSyntax  opt_networkTransferSyntax = EXS_Unknown;
E_GrpLenEncoding  opt_groupLength = EGL_recalcGL;
E_EncodingType    opt_sequenceType = EET_ExplicitLength;
E_PaddingEncoding opt_paddingType = EPD_withoutPadding;
int               opt_helpString = 0;
int               opt_verbose = 0;
int               opt_debug = 0;
int               opt_failInvalidQuery = 1;

static OFBool     opt_singleProcess = OFFalse;
#ifndef NODISPLAY
 static OFBool    opt_disableDisplay = OFTrue;
#endif
static int        opt_maxAssociations = 20;	/* default */

static void
shortusage()
{
    fprintf(stderr, 
"usage: %s [-c <db-path>][<options>] port\n", progname);
}


static void
fullusage()
{
    fprintf(stderr, "%s\n\n", rcsid);
    shortusage();
    fprintf(stderr,
"parameters:\n"
"    port	 tcp/ip port number to listen on\n"
#ifndef HAVE_FORK
"           This version allows only single process mode\n"
#endif
"  database options:\n"
"      -c <db-path> base path of worklist database\n"
"  network options:\n"
"      -r      refuse association\n"
"      +R      reject association if implementation class UID is missing\n"
"      -s n    sleep n seconds after find (default: 0)\n"
"      +s n    sleep n seconds during find (default: 0)\n"
"      -b n    set max receive pdu to n bytes (%lu-%lu, default: %lu)\n"
"      -t=     prefer local byte ordering, explicit transfer syntaxes (default)\n"
"      -te     prefer little-endian explicit transfer syntax\n"
"      -tb     prefer big-endian explicit transfer syntax\n"
"      -ti     use little-endian implicit transfer syntax only\n"
"    group length encoding (when sending C-FIND response data):\n"
"      +g      write with group lengths\n"
"      +g=     recalculate group lengths (default)\n"
"      -g      write without group lengths\n"
"    length encoding in sequences and items (when sending C-FIND response data):\n"
"      +e      write with explicit lengths (default)\n"
"      -e      write with undefined lengths\n"
"    unknown VR (when sending C-FIND response data):\n"
"      -u      disable generation of new VRs (UN/UT)\n"
"      +u      enable generation of new VRs (UN/UT) (default)\n"
"  other options:\n"
"      -h      print this usage string\n"
"      +V      verbose mode, print actions\n"
"      +d      debug mode\n"
#ifdef HAVE_FORK
"      -S      single process\n"
#endif
"      +D      enable display\n"
"      -f      disable failure status on invalid query (default: fail)\n",
    (unsigned long)ASC_MINIMUMPDUSIZE, 
    (unsigned long)ASC_MAXIMUMPDUSIZE, 
    (unsigned long)ASC_DEFAULTMAXPDU);
    exit(1);
}

static void 
usage()
{
    shortusage();
    exit(1);
}

void errmsg(const char* msg, ...)
{
    va_list args;
	
    fprintf(stderr, "%s: ", progname);
    va_start(args, msg);
    vfprintf(stderr, msg, args);
    va_end(args);
    fprintf(stderr, "\n");
}

static CONDITION waitForAssociation(T_ASC_Network *net);
static void cleanChildren();
static CONDITION negotiateAssociation(T_ASC_Association *assoc);

static int extract_unsigned_long(unsigned long& result, int argc, char *argv[], int idx)
{
  unsigned long l;
  
  if (idx >= argc ) return 0;
  if (1 == sscanf(argv[idx], "%lu", &l))
  {
    result = l;
    return 1;
  } else return 0;
}

int 
main(int argc, char *argv[])
{
    CONDITION cond;
    T_ASC_Network *net;
    int i;

#ifdef HAVE_GUSI_H
    /* needed for Macintosh */
    GUSISetup(GUSIwithSIOUXSockets);
    GUSISetup(GUSIwithInternetSockets);
#endif

#ifdef HAVE_WINSOCK_H
    WSAData winSockData;
    /* we need at least version 1.1 */
    WORD winSockVersionNeeded = MAKEWORD( 1, 1 );
    WSAStartup(winSockVersionNeeded, &winSockData);
#endif

    prepareCmdLineArgs(argc, argv, "wlistctn");

    /* strip any leading path from program name */
    if ((progname = (char *)strrchr(argv[0], PATH_SEPARATOR)) != NULL) {
	progname++;
    } else {
	progname = argv[0];
    }

    if (argc < 2) {
	fullusage();
    }

    /*
    ** Don't let dcmdata remove tailing blank padding or perform other
    ** manipulations.  We want to see the real data.
    */
    dcmEnableAutomaticInputDataCorrection = OFFalse;
    fprintf(stdout, "NOTICE: Dcmdata auto correction disabled!!\n");

    /* parse program arguments */

    
    dcmEnableUnknownVRGeneration = OFTrue;
    char *arg = NULL;
    for (i=1; i<argc; i++) {
        arg = argv[i];
        if (arg[0] == '-' || arg[0] == '+') {
            if (strlen(arg) < 2) {
                fprintf(stderr, "unknown argument: %s\n", arg);
                usage();
                return 1;
            }
            switch (arg[1]) {
    
	    case 'c':
	        if (i++ < argc) {
		    baseWorklistDBPath = argv[i];
                } else {
                    fprintf(stderr, "missing worklist database path\n");
		    usage();
                    return 1;
                }
	        break;
            case 'r': /* -r */
                if ((arg[0]=='-')&&(arg[2]=='\0')) {
                    opt_refuseAssociation = 1;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'R': /* +R */
                if ((arg[0]=='+')&&(arg[2]=='\0')) {
                    opt_rejectWithoutImplementationUID = 1;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'h': /* -h */
                if ((arg[0]=='-')&&(arg[2]=='\0')) {
                    opt_helpString = 1;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'V': /* +V */
                if ((arg[0]=='+')&&(arg[2]=='\0')) {
                    opt_verbose = 1;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'd': /* +d */
                if ((arg[0]=='+')&&(arg[2]=='\0')) {
                    opt_debug = 3;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 's': /* -s n */
                if (arg[2]=='\0') {
                    if (arg[0]=='-') {
                        if (extract_unsigned_long(opt_sleepAfterFind, argc, argv, i+1)) i++;
                        else {
                            fprintf(stderr, "illegal parameter in %s option\n", arg);
                            return 1;
                        }
                    } else {
                        if (extract_unsigned_long(opt_sleepDuringFind, argc, argv, i+1)) i++;
                        else {
                            fprintf(stderr, "illegal parameter in %s option\n", arg);
                            return 1;
                        }
                    }
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'b': /* -b n */
                if ((arg[0]=='-')&&(arg[2]=='\0')&&
                    (extract_unsigned_long(opt_maxPDU, argc, argv, i+1))) {
                    if ((opt_maxPDU<ASC_MINIMUMPDUSIZE)||(opt_maxPDU>ASC_MAXIMUMPDUSIZE)) {
                        fprintf(stderr, "option %s: PDU size out of range (%lu-%lu)\n", 
                            arg, (unsigned long)ASC_MINIMUMPDUSIZE, (unsigned long)ASC_MAXIMUMPDUSIZE);
                        return 1;
                    } else i++;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'u': /* +u, -u */
                if (arg[2] == '\0') {
                    if (arg[0] == '-') {
    		    dcmEnableUnknownVRGeneration = OFFalse;
    		    dcmEnableUnlimitedTextVRGeneration = OFFalse;
    	        } else {
    		    dcmEnableUnknownVRGeneration = OFTrue;
    		    dcmEnableUnlimitedTextVRGeneration = OFTrue;
    	        }
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'e': /* +e, -e */
                if (arg[2] == '\0') {
                    if (arg[0] == '-') opt_sequenceType=EET_UndefinedLength; 
                    else opt_sequenceType=EET_ExplicitLength;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                /* how should DIMSE encodes C-FIND-RSP data */
                g_dimse_send_sequenceType_encoding = opt_sequenceType;
                break;
            case 't': /* +t=, +ti, +te, +tb, -t=, -ti, -te, -tb */ 
                if ((strlen(arg)==3)&&(arg[0]=='-')) {
                    switch (arg[2]) {
                    case '=':
                        opt_networkTransferSyntax = EXS_Unknown;
                        break;
                    case 'i':
                        opt_networkTransferSyntax = EXS_LittleEndianImplicit;
                        break;
                    case 'e':
                        opt_networkTransferSyntax = EXS_LittleEndianExplicit;
                        break;
                    case 'b':
                        opt_networkTransferSyntax = EXS_BigEndianExplicit;
                        break;
                    default:
                        fprintf(stderr, "unknown option: %s\n", arg);
                        return 1;
                    }
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                break;
            case 'g': /* +g, +g=, -g */
                if (strcmp(arg,"+g")==0) {
                    opt_groupLength = EGL_withGL;
                } else if (strcmp(arg,"+g=")==0) {
                    opt_groupLength = EGL_recalcGL;
                } else if (strcmp(arg,"-g")==0) {
                    opt_groupLength = EGL_withoutGL;
                } else {
                    fprintf(stderr, "unknown option: %s\n", arg);
                    return 1;
                }
                /* how should DIMSE encode C-FIND-RSP data */
                g_dimse_send_groupLength_encoding = opt_groupLength;
                break;
        
            case 'D':
#ifdef NODISPLAY
    	        fprintf(stderr, "Warning: not compiled for ctndisplay\n");
#else
    	        opt_disableDisplay = OFFalse;
#endif	    
    	        break;
    
#ifdef HAVE_FORK
    	    case 'S':
    	        opt_singleProcess = OFTrue;
    	        break;
#endif

            case 'f':
    	        opt_failInvalidQuery = OFFalse;
    	        break;
                
            default:
                fprintf(stderr, "unknown option: %s\n", arg);
                return 1;
            } /* end switch */
       
        } else if (opt_port == 0)  {
            if ((! extract_unsigned_long(opt_port, argc, argv, i)) ||
                (opt_port > 0xFFFF) || (opt_port == 0)) {
                fprintf(stderr, "illegal port number: %s\n", argv[i]);
                return 1;
            }
        } else {
            fprintf(stderr, "too many arguments: %s\n", arg);
            usage();
            return 1;
        }
    }

    if (opt_helpString) {
        usage();
        return 0;
    }

    if (opt_port == 0) {
        fprintf(stderr, "missing port number\n");
        return 1;
    }

    DUL_Debug(opt_debug);
    DIMSE_debug(opt_debug);
    SetDebugLevel(((opt_debug>=3)?opt_debug:0));	/* dcmdata debugging */


#ifdef USE_MALLOC_DEBUG
#ifdef HAVE_MALLOC_DEBUG
#define MALLOCDEBUGLEVEL 16
    if (verbose) {
        printf("malloc_debug on (%d)\n", MALLOCDEBUGLEVEL);
    }
    malloc_debug(MALLOCDEBUGLEVEL);
#endif
#endif

#ifndef HAVE_FORK
    opt_singleProcess = OFTrue;
#endif

    if (access(baseWorklistDBPath, R_OK) < 0) {
	errmsg("cannot access base worklist db directory");
	errmsg("%s: %s", baseWorklistDBPath, strerror(errno));
	usage();
    }
    
    /* make sure data dictionary is loaded */
    if (!dcmDataDict.isDictionaryLoaded()) {
	fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n",
		DCM_DICT_ENVIRONMENT_VARIABLE);
    }
    
    processTable.pcnt = 0;
    processTable.plist = NULL;

#ifdef HAVE_GETEUID
    /* if port is privileged we must be as well */
    if (opt_port < 1024) {
        if (geteuid() != 0) {
	    errmsg("cannot listen on port %d, insufficient privileges", opt_port);
	    usage();
	}
    }
#endif
    
    cond = ASC_initializeNetwork(NET_ACCEPTOR, (int)opt_port, 1000, &net);
    if (!SUCCESS(cond)) {
	errmsg("Error initialising network:");
	COND_DumpConditions();
	exit(1);
    }

#if defined(HAVE_SETUID) && defined(HAVE_GETUID)
    /* return to normal uid so that we can't do too much damage in case
     * things go very wrong.   Only works if the program is setuid root,
     * and run by another user.  Running as root user may be
     * potentially disasterous if this program screws up badly.
     */
    setuid(getuid());
#endif

    /* initialise display interface */
#ifndef NODISPLAY
    DSU_setTraceLevel(opt_debug);
    if (opt_disableDisplay) {
        DSU_setDisplayMode(OFFalse);
    } else {
        DSU_setDisplayMode(OFTrue);
    }
    DSU_initDisplay(DSU_CTNUser);
#endif

    /* loop waiting for associations */
    while (SUCCESS(cond)) {

	cond = waitForAssociation(net);

	if (!opt_singleProcess) {
	    cleanChildren();	/* clean up any child processes */
	}
    }

    cond = ASC_dropNetwork(&net);
    if (!SUCCESS(cond)) {
	errmsg("Error dropping network:");
	COND_DumpConditions();
	exit(1);
    }

#ifdef HAVE_WINSOCK_H
    WSACleanup();
#endif
    
    return 0;

}

/*
 * Process Table List Handling
 */

static void
addProcessToTable(int pid, T_ASC_Association * assoc)
{
    CTN_ProcessSlot	*ps;

    if ((ps = (CTN_ProcessSlot*)malloc(sizeof(CTN_ProcessSlot))) == NULL) {
	errmsg("out of memory");
	return;
    }
    /* fill in info */
    ASC_getPresentationAddresses(assoc->params, ps->peerName, NULL);
    ASC_getAPTitles(assoc->params, ps->callingAETitle, 
	ps->calledAETitle, NULL);
    ps->processId = pid;
    ps->startTime = time(NULL);
    ps->hasStorageAbility = OFFalse;
    /* add to start of list */
    processTable.pcnt++;
    ps->next = processTable.plist;
    processTable.plist = ps;
}

#if 0
/* Unused */
static CTN_ProcessSlot* procWithCalledAETitle(const char* aeTitle)
{
    CTN_ProcessSlot	*ps;
    OFBool		found = OFFalse;

    if (aeTitle == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found) {
	found = strcmp(ps->calledAETitle, aeTitle) == 0;
	if (!found) ps = ps->next;
    }
    return ps;
}

static CTN_ProcessSlot* procWithCallingAETitle(const char* aeTitle)
{
    CTN_ProcessSlot	*ps;
    OFBool		found = OFFalse;

    if (aeTitle == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found) {
	found = strcmp(ps->callingAETitle, aeTitle) == 0;
	if (!found) ps = ps->next;
    }
    return ps;
}
#endif

#if 0
/* currently unused */
static int
countProcessesForCalledAE(const char* aeTitle)
{
    int n = 0;
    CTN_ProcessSlot	*ps;

    if (aeTitle == NULL) return 0;

    ps = processTable.plist;
    while (ps) {
	if (strcmp(ps->calledAETitle, aeTitle) == 0) {
	    n++;
	}
	ps = ps->next;
    }
    return n;
}
#endif

#if 0
/* Unused */
static CTN_ProcessSlot* procWithPeerName(const char* peerName)
{
    CTN_ProcessSlot	*ps;
    OFBool		found = OFFalse;

    if (peerName == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found) {
	found = strcmp(ps->peerName, peerName) == 0;
	if (!found) ps = ps->next;
    }
    return ps;
}
#endif

static int
countChildProcesses()
{
    int n = 0;
    CTN_ProcessSlot	*ps;

    ps = processTable.plist;
    while (ps) {
	n++;
	ps = ps->next;
    }
    return n;
}

static void
removeProcessFromTable(int pid)
{
    CTN_ProcessSlot	*ps, *pps;
    OFBool		found = OFFalse;

    pps = NULL;
    ps = processTable.plist;

    while (ps && !found) {
	found = (ps->processId == pid);
	if (!found) {
	    pps = ps;
	    ps = ps->next;
	}
    }

    if (!found) {
	errmsg("WARNING: process %d not found", pid);
	return;
    }

    if (pps == NULL) {
	/* first in list */
	processTable.plist = ps->next;
    } else {
	pps->next = ps->next;
    }

    free(ps);
    processTable.pcnt--;
}

#ifdef HAVE_WAITPID
static void 
cleanChildren()
{
    int                 options;
    int			stat_loc;
    int                 child = 1;

    options = WNOHANG;
    while (child > 0) {
	child = (int)(waitpid(-1, &stat_loc, options));
	if (child == 0) {
	    /* child not yet finished so don't complain */
	} else if (child < 0) {
	    if (errno == ECHILD) {
		/* no children so don't complain */
	    } else {
		errmsg("wait for child failed: %s", strerror(errno));
	    }
	} else if (child > 0) {
	    if (opt_verbose) {
		printf("Cleaned up after child (%d)\n", child);
	    }
	/* Remove Entry from Process Table */
	    removeProcessFromTable(child);
	}
    }
}
#elif HAVE_WAIT3
static void 
cleanChildren()
{
#if defined(__NeXT__)
    /* some systems need a union wait as argument to wait3 */
    union wait          status;
#else
    int                 status;
#endif
    int                 options;
    struct rusage       rusage;
    int                 child = 1;

    options = WNOHANG;
    while (child > 0) {
	child = wait3(&status, options, &rusage);
	if (child < 0) {
	    if (errno == ECHILD) {
		/* no children so don't complain */
	    } else {
		errmsg("wait for child failed: %s", strerror(errno));
	    }
	} else if (child > 0) {
	    if (opt_verbose) {
		printf("Cleaned up after child (%d)\n", child);
	    }
	/* Remove Entry from Process Table */
	    removeProcessFromTable(child);
	}
    }
}
#else
/* don't know how to wait for my children */
static void 
cleanChildren()
{
    errmsg("cannot wait for child processes");
}
#endif


/*
 * Refuse an Association
 */

static CONDITION
refuseAssociation(T_ASC_Association ** assoc, CTN_RefuseReason reason)
{
    CONDITION           cond;
    T_ASC_RejectParameters rej;

    if (opt_verbose) {
	printf("Refusing Association (");
	switch (reason) {
	case CTN_TooManyAssociations:
	    printf("TooManyAssociations");
	    break;
	case CTN_CannotFork:
	    printf("CannotFork");
	    break;
	case CTN_BadAppContext:
	    printf("BadAppContext");
	    break;
	case CTN_BadAEPeer:
	    printf("BadAEPeer");
	    break;
	case CTN_BadAEService:
	    printf("BadAEService");
	    break;
	case CTN_NoReason:
	    printf("NoReason");
	    break;
	default:
	    printf("???");
	    break;
	}
	printf(")\n");
    }
    switch (reason) {
    case CTN_TooManyAssociations:
	rej.result = ASC_RESULT_REJECTEDTRANSIENT;
	rej.source = ASC_SOURCE_SERVICEPROVIDER_PRESENTATION_RELATED;
	rej.reason = ASC_REASON_SP_PRES_LOCALLIMITEXCEEDED;
	break;
    case CTN_CannotFork:
	rej.result = ASC_RESULT_REJECTEDPERMANENT;
	rej.source = ASC_SOURCE_SERVICEPROVIDER_PRESENTATION_RELATED;
	rej.reason = ASC_REASON_SP_PRES_TEMPORARYCONGESTION;
	break;
    case CTN_BadAppContext:
	rej.result = ASC_RESULT_REJECTEDTRANSIENT;
	rej.source = ASC_SOURCE_SERVICEUSER;
	rej.reason = ASC_REASON_SU_APPCONTEXTNAMENOTSUPPORTED;
	break;
    case CTN_BadAEPeer:
	rej.result = ASC_RESULT_REJECTEDPERMANENT;
	rej.source = ASC_SOURCE_SERVICEUSER;
	rej.reason = ASC_REASON_SU_CALLINGAETITLENOTRECOGNIZED;
	break;
    case CTN_BadAEService:
	rej.result = ASC_RESULT_REJECTEDPERMANENT;
	rej.source = ASC_SOURCE_SERVICEUSER;
	rej.reason = ASC_REASON_SU_CALLEDAETITLENOTRECOGNIZED;
	break;
    case CTN_NoReason:
    default:
	rej.result = ASC_RESULT_REJECTEDPERMANENT;
	rej.source = ASC_SOURCE_SERVICEUSER;
	rej.reason = ASC_REASON_SU_NOREASON;
	break;
    }

    cond = ASC_rejectAssociation(*assoc, &rej);
    if (cond != ASC_NORMAL) {
	fprintf(stderr, "Association Reject Failed:\n");
	COND_DumpConditions();
    }

    cond = ASC_dropAssociation(*assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Drop Association:\n");
	COND_DumpConditions();
    }
    cond = ASC_destroyAssociation(assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Destroy Association:\n");
	COND_DumpConditions();
    }

    return cond;
}

/*
** Return true if path is a directory 
*/

static OFBool
isaDirectory(const char* path)
{
    OFBool ok = OFFalse;
#ifdef HAVE__FINDFIRST
    struct _finddata_t fileData;
    long hFile;
    if (((hFile = _findfirst(path, &fileData)) != -1L) && 
        (fileData.attrib & _A_SUBDIR)) {
        /* it is a directory */
	_findclose(hFile);
        ok = OFTrue;
    }
#else
    DIR *d = NULL;
    if ((d = opendir(path)) != NULL) {
	ok = OFTrue;
	closedir(d);
    }
#endif
    return ok;
}

/*
 * Wait for and negotiate incoming association.
 */

static CONDITION
waitForAssociation(T_ASC_Network * net)
{
    CONDITION           cond;
#ifdef HAVE_FORK
    int                 pid;
#endif
    T_ASC_Association  *assoc;
    char                buf[BUFSIZ];
    int timeout;

    if (opt_singleProcess) {
	timeout = 1000;
    } else {
	if (countChildProcesses() > 0) {
	    timeout = 1;
	} else {
	    timeout = 1000;
	}
    }
    if (ASC_associationWaiting(net, timeout)) {
        cond = ASC_receiveAssociation(net, &assoc, opt_maxPDU);
        if (!SUCCESS(cond)) {
	    if (opt_verbose) {
	        errmsg("Failed to receive association:");
	        COND_DumpConditions();
	    }
	    goto cleanup;
        }
    } else {
	return ASC_NORMAL;
    }

    if (opt_verbose) {
	printf("Association Received (%s:%s -> %s)\n",
	       assoc->params->DULparams.callingPresentationAddress,
	       assoc->params->DULparams.callingAPTitle,
	       assoc->params->DULparams.calledAPTitle);
    }
    if (opt_debug) {
	printf("Parameters:\n");
	ASC_dumpParameters(assoc->params);
    }
    if (opt_refuseAssociation) {
	if (opt_verbose) {
	    printf("Refusing Association (forced via command line)\n");
	}
	cond = refuseAssociation(&assoc, CTN_NoReason);
	goto cleanup;
    }

    /* Application Context Name */
    if ((cond = ASC_getApplicationContextName(assoc->params, buf) !=
	 ASC_NORMAL) || strcmp(buf, DICOM_STDAPPLICATIONCONTEXT) != 0) {
        /* reject: the application context name is not supported */
	if (opt_verbose) {
	    errmsg("Bad AppContextName: %s", buf);
	}
	cond = refuseAssociation(&assoc, CTN_BadAppContext);
	goto cleanup;
    }

    /* Implementation Class UID */
    if (opt_rejectWithoutImplementationUID &&
	strlen(assoc->params->theirImplementationClassUID) == 0) {
        /* reject: no implementation Class UID provided */
	errmsg("Refusing Association: No implementation Class UID provided");
	cond = refuseAssociation(&assoc, CTN_NoReason);
	goto cleanup;
    }


    /* too many concurrent associations ?? 
     */
    if (countChildProcesses() >= opt_maxAssociations) {
	cond = refuseAssociation(&assoc, CTN_TooManyAssociations);
	goto cleanup;
    }

    /* is the called AE title known (which worklist to query)? */
    {
	char dbPath[2048];
	sprintf(dbPath, "%s/%s", baseWorklistDBPath, 
		assoc->params->DULparams.calledAPTitle);
	if (!isaDirectory(dbPath)) {
	    /* there is no subdirectory for this AE Title */
	    cond = refuseAssociation(&assoc, CTN_BadAEService);
	    goto cleanup;
	}
    }

    cond = negotiateAssociation(assoc);
    if (cond != ASC_NORMAL) {
	goto cleanup;
    }

    cond = ASC_acknowledgeAssociation(assoc);
    if (cond != ASC_NORMAL) {
	COND_DumpConditions();
	goto cleanup;
    }
    if (opt_verbose) {
	printf("Association Acknowledged (Max Send PDV: %lu)\n",
	       assoc->sendPDVLength);
	if (ASC_countAcceptedPresentationContexts(assoc->params) == 0)
	    printf("    (but no valid presentation contexts)\n");
	if (opt_debug)
	    ASC_dumpParameters(assoc->params);
    }


    if (opt_singleProcess) {
        /* don't spawn a sub-process to handle the association */
	cond = SCE_handleAssociation(assoc);
    } 
#ifdef HAVE_FORK
      else {
        /* spawn a sub-process to handle the association */
	pid = (int)(fork());
	if (pid < 0) {
	    errmsg("Cannot create association sub-process: %s",
		   strerror(errno));
	    cond = refuseAssociation(&assoc, CTN_CannotFork);
	    goto cleanup;
	} else if (pid > 0) {
	    /* parent process, note process in table */

	    addProcessToTable(pid, assoc);

	} else {
	    /* child process, handle the association */
	    cond = SCE_handleAssociation(assoc);
	    /* the child process is done so exit */
	    exit(0);
	}
    }
#endif

cleanup:
    if (!opt_singleProcess) {
        /* the child will handle the association, we can drop it */
        cond = ASC_dropAssociation(assoc);
        if (!SUCCESS(cond)) {
	   errmsg("Cannot Drop Association:");
	   COND_DumpConditions();
        }
        cond = ASC_destroyAssociation(&assoc);
        if (!SUCCESS(cond)) {
	    errmsg("Cannot Destroy Association:");
	    COND_DumpConditions();
        }
    }

    if (opt_verbose) {
        /* are there any conditions sitting on the condition stack? */
	char                buf2[BUFSIZ];
	CONDITION           c;

	if (COND_TopCondition(&c, buf2, BUFSIZ) != COND_NORMAL) {
	    printf("CONDITIONS Remaining:");
	    COND_DumpConditions();
	}
    }
    COND_PopCondition(OFTrue);	/* clean out condition stack */
    return cond;
}


static CONDITION
negotiateAssociation(T_ASC_Association * assoc)
{
    CONDITION cond = ASC_NORMAL;
    DIC_AE calledAETitle;

    ASC_getAPTitles(assoc->params, NULL, calledAETitle, NULL);
  
    const char* transferSyntaxes[] = { NULL, NULL, NULL };
    int numTransferSyntaxes = 0;
  
    switch (opt_networkTransferSyntax)
    {
    case EXS_LittleEndianImplicit:
        /* we only support Little Endian Implicit */
        transferSyntaxes[0]  = UID_LittleEndianImplicitTransferSyntax;
        numTransferSyntaxes = 1;
        break;
    case EXS_LittleEndianExplicit:
        /* we prefer Little Endian Explicit */
        transferSyntaxes[0] = UID_LittleEndianExplicitTransferSyntax;
        transferSyntaxes[1] = UID_LittleEndianImplicitTransferSyntax;
        numTransferSyntaxes = 2;
        break;
    case EXS_BigEndianExplicit:
        /* we prefer Big Endian Explicit */
        transferSyntaxes[0] = UID_BigEndianExplicitTransferSyntax;
        transferSyntaxes[1] = UID_LittleEndianImplicitTransferSyntax;
        numTransferSyntaxes = 2;
        break;
  
    default:
        /* We prefer explicit transfer syntaxes. 
         * If we are running on a Little Endian machine we prefer 
         * LittleEndianExplicitTransferSyntax to BigEndianTransferSyntax.
         */
        if (gLocalByteOrder == EBO_LittleEndian)  /* defined in dcxfer.h */
        {
          transferSyntaxes[0] = UID_LittleEndianExplicitTransferSyntax;
          transferSyntaxes[1] = UID_BigEndianExplicitTransferSyntax;
        } else {
          transferSyntaxes[0] = UID_BigEndianExplicitTransferSyntax;
          transferSyntaxes[1] = UID_LittleEndianExplicitTransferSyntax;
        }
        transferSyntaxes[2] = UID_LittleEndianImplicitTransferSyntax;
        numTransferSyntaxes = 3;
        break;
    }


    /*  accept any of the supported abstract syntaxes */
    cond = ASC_acceptContextsWithPreferredTransferSyntaxes(
	assoc->params, 
	(const char**)supportedAbstractSyntaxes, numberOfSupportedAbstractSyntaxes,
	(const char**)transferSyntaxes, DIM_OF(transferSyntaxes));
    if (!SUCCESS(cond)) {
	errmsg("Cannot accept presentation contexts:");
	COND_DumpConditions();
    }

    return cond;
}


/*
** CVS Log
** $Log: wlistctn.cc,v $
** Revision 1.23  2000/03/08 16:37:45  meichel
** Updated copyright header.
**
** Revision 1.22  2000/02/29 11:59:33  meichel
** Removed support for VS value representation. This was proposed in CP 101
**   but never became part of the standard.
**
** Revision 1.21  2000/02/23 15:13:57  meichel
** Corrected macro for Borland C++ Builder 4 workaround.
**
** Revision 1.20  2000/02/02 15:18:32  meichel
** Replaced some #if statements by more robust #ifdef
**
** Revision 1.19  2000/02/01 11:00:59  meichel
** Avoiding to include <stdlib.h> as extern "C" on Borland C++ Builder 4,
**   workaround for bug in compiler header files.
**
** Revision 1.18  1999/05/03 14:14:46  joergr
** Minor code purifications to keep Sun CC 2.0.1 quiet.
**
** Revision 1.17  1999/04/26 12:59:12  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.16  1998/07/24 13:30:20  meichel
** Minor changes to avoid compiler warnings (gcc 2.8.1 with additional
**   options), e.g. add copy constructors.
**
** Revision 1.15  1998/02/06 15:07:43  meichel
** Removed many minor problems (name clashes, unreached code)
**   reported by Sun CC4 with "+w" or Sun CC2.
**
** Revision 1.14  1998/01/14 14:27:27  hewett
** Modified existing -u command line option to also disable generation
** of UT and VS (previously just disabled generation of UN).
**
** Revision 1.13  1997/09/18 10:09:12  meichel
** Removed unused (and undocumented) option -XF from wlistctn.
**
** Revision 1.12  1997/09/18 08:11:06  meichel
** Many minor type conflicts (e.g. long passed as int) solved.
**
** Revision 1.11  1997/07/21 09:08:52  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.10  1997/07/04 09:25:00  meichel
** Simplified some sizeof() constructs to avoid compiler warnings
**   on the IBM xlC compiler (AIX 3.x).
**
** Revision 1.9  1997/05/29 17:06:40  meichel
** All dcmtk applications now contain a version string
** which is displayed with the command line options ("usage" message)
** and which can be queried in the binary with the "ident" command.
**
** Revision 1.8  1997/05/22 13:35:57  hewett
** Modified the test for presence of a data dictionary to use the
** method DcmDataDictionary::isDictionaryLoaded().
**
** Revision 1.7  1997/03/27 16:11:50  hewett
** Added command line switches allowing generation of UN to
** be disabled (it is enabled by default).
**
** Revision 1.6  1996/09/27 08:54:27  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C
**
** Revision 1.5  1996/09/24 15:49:54  hewett
** Added conditionaly compiled support for the Macintosh
** environment (GUSI library).
**
** Revision 1.4  1996/08/29 16:16:10  hewett
** Corrected copyright header.
**
** Revision 1.3  1996/05/31 13:50:44  hewett
** Now checks that the called AE Title corresponds to a Worklist database
** directory.  If no such directory exists then the association is rejected
** due to an unknown AE Title.
**
** Revision 1.2  1996/05/01 17:33:50  hewett
** Modified for latest Worklist DB.
**
** Revision 1.1.1.1  1996/04/26 14:16:01  hewett
** Initial Import.
**
*/
